##############################################################################
#
# Print billing management system - support modules, version 4.0.6
#
# Copyright (C) 2000, 2001, 2002 Daniel Franklin
#
# This module is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself.
#
# This module implements parsers to get useful stuff out of a configuration
# file. If the configuration file only contains a URL, we attempt to use
# wget to download the configuration file from the specified location.
#
##############################################################################

package Printbill::printbill_pcfg;

use POSIX;
use strict;

BEGIN {
	use Exporter ();
	
	@Printbill::printbill_pcfg::ISA = qw (Exporter);
	@Printbill::printbill_pcfg::EXPORT = qw (&pcfg &pcfg_printers);
	$Printbill::printbill_pcfg::VERSION = 4.0.6;
}


my $WGET = '/usr/bin/wget';

# Do a preliminary check on the config file - if it just redirects to a URL
# then suck down the config file using wget. Otherwise everything is local -
# return undefined.

sub check_remote_cfg
{
	my ($config) = @_;
	my ($newconfig, $line, $my_pid);
	
	if (defined ($config) && -r $config) {
		open (CONFIGFILE, $config) or do {
			print STDERR "$0 (printbill_pcfg::check_remote_cfg()): unable to open configuration file $config: $!\n";
			return undef;
		};
	} else {
		return undef;
	}
	
	$line = <CONFIGFILE>;
	chomp $line;
	
# Check if the first line is a URL.

	if ($line =~ /^(ht|f)tp:\/\//) {
# If so, suck the target down using wget
		if (-x $WGET) {
			`$WGET $line -Y off -O /tmp/printbillrc_$$ &> /dev/null`;

# Success? Send the new filename back.

			if ($? == 0) {
				return "/tmp/printbillrc_$$";
			} else {
				return undef;
			}
		} else {
			print " 
Sorry - couldn't find $WGET. Normally I'd put this sort of thing in a
config file, but this is the script used to read the config file and so the
path must be hard-coded (ye olde chicken and egg problem). To fix it so that
we can find wget, please edit the file printbill_pcfg.pm and change the line
at the top which says \$WGET = \"$WGET\" - or make a symlink.\n\n";
			
			return undef;
		}
	} else {
# Not remote.
		return undef;
	}
	
	close CONFIGFILE
		|| die "$0 (printbill_pcfg::check_remote_cfg()): unable to close config file $config: $!\n";
}

# Parse configuration file
#
# 1st arg: full path to configuration file
#
# 2nd arg: optional printer specification. If not provided, default values
# for prices etc. are returned. We even cater for Americans who don't speak
# English ;-)

sub pcfg
{
	my ($config, $printer) = @_;

	if (!(defined $printer)) {
		$printer = '';
	}
	
# Remember, perl initialises variables to zero or ().

	my ($remote_config, %params, $line, $subline, @strings, @pstr, $defprinter);
	
	$defprinter = 0;
	
	$remote_config = check_remote_cfg ($config);
	
	if (defined ($remote_config)) {
		open (CONFIGFILE, $remote_config) or do {
			print STDERR "$0 (printbill_pcfg.pm): unable to open configuration file $config: $!\n";
			return ();
		};
	} elsif (-r $config) {
		open (CONFIGFILE, $config) or do {
			print STDERR "$0 (printbill_pcfg.pm): unable to open configuration file $config: $!\n";
			return ();
		};
	} else {
		return ();
	}
	
# Config file errors are ignored. The last valid entries for prices are
# accepted.
	
	while (defined ($line = <CONFIGFILE>)) {
		chomp $line;
		
		next if ($line =~ /^\#.*/ || $line eq "");
		
		$line =~ /^(.+)$/;
		$line = $1;

		@strings = split (/\s*:\s*/, $line);

# Colons are allowed in strings [>=1]

		if ($#strings >= 1) {
			if ($defprinter == 0 && !defined $params{'colourspace'} && ($strings [0] eq 'def_colourspace' || $strings [0] eq 'def_colorspace')) {
				$params{'colourspace'} = $strings [1];
			} elsif ($defprinter == 0 && !defined $params{'price_per_percent_black'} && $strings [0] eq 'def_price_per_percent_black') {
				$params{'price_per_percent_black'} = $strings [1];
			} elsif ($defprinter == 0 && !defined $params{'price_per_percent_colour'} && ($strings [0] eq 'def_price_per_percent_colour' || $strings [0] eq 'def_price_per_percent_color')) {
				$params{'price_per_percent_colour'} = $strings [1];
				$params{'price_per_percent_cyan'} = $strings [1];
				$params{'price_per_percent_magenta'} = $strings [1];
				$params{'price_per_percent_yellow'} = $strings [1];
			} elsif ($defprinter == 0 && !defined $params{'price_per_page'} && $strings [0] eq 'def_price_per_page') {
				$params{'price_per_page'} = $strings [1];
			} elsif ($defprinter == 0 && !defined $params{'estimated_total_percent_black'} && $strings [0] eq 'def_estimated_total_percent_black') {
				$params{'estimated_total_percent_black'} = $strings [1];
			} elsif ($defprinter == 0 && !defined $params{'estimated_total_percent_colour'} && ($strings [0] eq 'def_estimated_total_percent_colour' || $strings [0] eq 'def_estimated_total_percent_color')) {
				$params{'estimated_total_percent_colour'} = $strings [1];
			} elsif ($defprinter == 0 && !defined $params{'response_method'} && $strings [0] eq 'def_response_method') {
				$params{'response_method'} = $strings [1];
			} elsif ($strings [0] eq 'db_home') {
				$params{'db_home'} = $strings [1];

				for (my $i = 2; $i <= $#strings; $i++) {
					$params{'db_home'} .= (":" . $strings [$i]);
				}

				chomp $params{'db_home'};
			} elsif ($strings [0] eq 'currency_symbol') {
				$params{'currency_symbol'} = $strings [1];
			} elsif ($strings [0] eq 'printbilld_user' || $strings [0] eq 'lpd_user') {
				$params{'printbilld_user'} = $strings [1];
			} elsif ($strings [0] eq 'printbilld_group') {
				$params{'printbilld_group'} = $strings [1];
			} elsif ($strings [0] eq 'web_user') {
				$params{'web_user'} = $strings [1];
			} elsif ($strings [0] eq 'printcap') {
				$params{'printcap'} = $strings [1];
			} elsif ($strings [0] eq 'bill_process_method') {
				$params{'bill_process_method'} = $strings [1];
			} elsif ($strings [0] eq 'bill_max_processes') {
				$params{'bill_max_processes'} = $strings [1];
			} elsif ($strings [0] eq 'filter') {
				$params{'filter'} = $strings [1];
			} elsif ($strings [0] eq 'nice') {
				$params{'nice'} = $strings [1];
			} elsif ($strings [0] eq 'small_bill_niceness') {
				$params{'small_bill_niceness'} = $strings [1];
			} elsif ($strings [0] eq 'large_bill_niceness') {
				$params{'large_bill_niceness'} = $strings [1];
			} elsif ($strings [0] eq 'quote_generation_niceness') {
				$params{'quote_generation_niceness'} = $strings [1];
			} elsif ($strings [0] eq 'bill_nicethreshold') {
				$params{'bill_nicethreshold'} = $strings [1];
			} elsif ($strings [0] eq 'verbosity') {
				$params{'verbosity'} = $strings [1];
			} elsif ($strings [0] eq 'save_bad_path') {
				$params{'save_bad_path'} = $strings [1];
			} elsif ($strings [0] eq 'dpi') {
				$params{'dpi'} = $strings [1];
			} elsif ($strings [0] eq 'stats_path') {
				$params{'stats_path'} = $strings [1];
			} elsif ($strings [0] eq 'financelog') {
				$params{'financelog'} = $strings [1];
			} elsif ($strings [0] eq 'prog_path') {
				$params{'prog_path'} = $strings [1];
			} elsif ($strings [0] eq 'admin_prog_path') {
				$params{'admin_prog_path'} = $strings [1];
			} elsif ($strings [0] eq 'gs') {
				$params{'gs'} = $strings [1];
			} elsif ($strings [0] eq 'lpr') {
				$params{'lpr'} = $strings [1];
			} elsif ($strings [0] eq 'smbclient') {
				$params{'smbclient'} = $strings [1];
			} elsif ($strings [0] eq 'mta') {
				$params{'mta'} = $strings [1];
			} elsif ($strings [0] eq 'gnuplot') {
				$params{'gnuplot'} = $strings [1];
			} elsif ($strings [0] eq 'admin_mail') {
				$params{'admin_mail'} = $strings [1];
			} elsif ($strings [0] eq 'retry_interval') {
				$params{'retry_interval'} = $strings [1];
			} elsif ($strings [0] eq 'cookieroot') {
				$params{'cookieroot'} = $strings [1];
			} elsif ($strings [0] eq 'admin_cgi') {
				$params{'admin_cgi'} = $strings [1];
			} elsif ($strings [0] eq 'check_cgi') {
				$params{'check_cgi'} = $strings [1];
			} elsif ($strings [0] eq 'web_header') {
				$params{'web_header'} = $strings [1];
			} elsif ($strings [0] eq 'web_footer') {
				$params{'web_footer'} = $strings [1];
			} elsif ($strings [0] eq 'web_message_text') {
				$params{'web_message_text'} = $strings [1];
			} elsif ($strings [0] eq 'extra_quota_message') {
				$params{'extra_quota_message'} = $strings [1];
			} elsif ($strings [0] eq 'admin_form_type') {
				$params{'admin_form_type'} = $strings [1];
			} elsif ($strings [0] eq 'png_url') {
				$params{'png_url'} = $strings [1];
			} elsif ($strings [0] eq 'graph_png_output_dir') {
				$params{'graph_png_output_dir'} = $strings [1];
			} elsif ($strings [0] eq 'defineprinter' && $defprinter == 0) {
# If a printer definition exists, check to see if it matches the specified
# printer. The first one which matches is used.
	
				if ($strings [1] eq $printer) {
					$defprinter = 1;
					
					while (defined ($subline = <CONFIGFILE>) && ($subline !~ /end/i)) {
						@pstr = split (/\s*:\s*/, $subline);
						
						if ($#pstr == 1) {
							chomp $pstr [1];
							$pstr [0] =~ s/\s+//;
							
							if ($pstr [0] eq 'colourspace' || $pstr [0] eq 'colorspace') {
								$params{'colourspace'} = $pstr [1];
							} elsif ($pstr [0] eq 'price_per_percent_black') {
								$params{'price_per_percent_black'} = $pstr [1];
							} elsif ($pstr [0] eq 'price_per_percent_colour' || $pstr [0] eq 'price_per_percent_color') {
								$params{'price_per_percent_colour'} = $pstr [1];
								$params{'price_per_percent_cyan'} = $pstr [1];
								$params{'price_per_percent_magenta'} = $pstr [1];
								$params{'price_per_percent_yellow'} = $pstr [1];
							} elsif ($pstr [0] eq 'price_per_page') {
								$params{'price_per_page'} = $pstr [1];
							} elsif ($pstr [0] eq 'estimated_total_percent_black') {
								$params{'estimated_total_percent_black'} = $pstr [1];
							} elsif ($pstr [0] eq 'estimated_total_percent_colour') {
								$params{'estimated_total_percent_colour'} = $pstr [1];
							} elsif ($pstr [0] eq 'response_method') {
								$params{'response_method'} = $strings [1];
							}
						}
					}
				}
			}
		}
	}
	
# We may as well return the (apparently) valid data, even if these syscalls
# seem to have exploded.

	close CONFIGFILE or do {
		print STDERR "$0 (printbill_pcfg::pcfg): unable to close config file $config: $!\n";
	};
	
	if (defined ($remote_config)) {
		`/bin/rm $remote_config`;
		
		if ($? >> 8) {
			print STDERR "$0 (printbill_pcfg::pcfg_printers): unable to remove remote config file $remote_config: $!\n";
		}

	}

# Paranoia check - we make sure that if something is missing, the user knows about it.
	
	if (!defined $params{'colourspace'}) {
		print STDERR "Warning: missing entry \"def_colourspace\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'colourspace'} eq "unconfigured") {
		print STDERR "\"def_colourspace\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (defined $params{'colourspace'} && ($params{'colourspace'} eq "mono" || $params{'colourspace'} eq "colour" || $params{'colourspace'} eq "cmyk") && !defined $params{'price_per_percent_black'}) {
		print STDERR "Warning: missing entry \"def_price_per_percent_black\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'price_per_percent_black'} eq "unconfigured") {
		print STDERR "\"price_per_percent_black\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (defined $params{'colourspace'} && ($params{'colourspace'} eq "colour" || $params{'colourspace'} eq "cmyk" || $params{'colourspace'} eq "cmy")) {
		if (!defined $params{'price_per_percent_colour'}) {
			print STDERR "Warning: missing entry \"def_price_per_percent_colour\" in /etc/printbill/printbillrc\n";
		} elsif ($params{'price_per_percent_colour'} eq "unconfigured") {
			print STDERR "\"def_price_per_percent_colour\" not yet configured in /etc/printbill/printbillrc\n";
		}
	}

	if (!defined $params{'price_per_page'}) {
		print STDERR "Warning: missing entry \"def_price_per_page\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'price_per_page'} eq "unconfigured") {
		print STDERR "\"price_per_page\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'response_method'}) {
		print STDERR "Warning: missing entry \"response_method\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'response_method'} eq "unconfigured") {
		print STDERR "\"response_method\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (defined $params{'colourspace'} && ($params{'colourspace'} eq "mono" || $params{'colourspace'} eq "colour" || $params{'colourspace'} eq "cmyk") && !defined $params{'estimated_total_percent_black'}) {
		print STDERR "Warning: missing entry \"def_estimated_total_percent_black\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'estimated_total_percent_black'} eq "unconfigured") {
		print STDERR "\"estimated_total_percent_black\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (defined $params{'colourspace'} && ($params{'colourspace'} eq "colour" || $params{'colourspace'} eq "cmyk" || $params{'colourspace'} eq "cmy")) {
		if (!defined $params{'estimated_total_percent_colour'}) {
			print STDERR "Warning: missing entry \"def_estimated_total_percent_colour\" in /etc/printbill/printbillrc\n";
		} elsif ($params{'estimated_total_percent_colour'} eq "unconfigured") {
			print STDERR "\"estimated_total_percent_colour\" not yet configured in /etc/printbill/printbillrc\n";
		}
	}
	
	if (!defined $params{'db_home'}) {
		print STDERR "Warning: missing entry \"db_home\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'db_home'} eq "unconfigured") {
		print STDERR "\"db_home\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'prog_path'}) {
		print STDERR "Warning: missing entry \"prog_path\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'prog_path'} eq "unconfigured") {
		print STDERR "\"prog_path\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'admin_prog_path'}) {
		print STDERR "Warning: missing entry \"admin_prog_path\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'admin_prog_path'} eq "unconfigured") {
		print STDERR "\"admin_prog_path\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'dpi'}) {
		print STDERR "Warning: missing entry \"dpi\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'dpi'} eq "unconfigured") {
		print STDERR "\"dpi\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'gs'}) {
		print STDERR "Warning: missing entry \"gs\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'gs'} eq "unconfigured") {
		print STDERR "\"gs\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'lpr'}) {
		print STDERR "Warning: missing entry \"lpr\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'lpr'} eq "unconfigured") {
		print STDERR "\"lpr\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (defined $params{'smbclient'} && $params{'smbclient'} eq "unconfigured") {
		print STDERR "\"smbclient\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'printbilld_user'}) {
		print STDERR "Warning: missing entry \"printbilld_user\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'printbilld_user'} eq "unconfigured") {
		print STDERR "\"printbilld_user\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
# This one can be missing - we choose a sensible default

	if (defined $params{'printbilld_group'} && $params{'printbilld_group'} eq "unconfigured") {
		print STDERR "\"printbilld_group\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'filter'}) {
		$params{'filter'} = '/bin/cat';
	} elsif ($params{'filter'} eq "unconfigured") {
		print STDERR "\"filter\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'mta'}) {
		print STDERR "Warning: missing entry \"mta\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'mta'} eq "unconfigured") {
		print STDERR "\"mta\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'admin_mail'}) {
		print STDERR "Warning: missing entry \"admin_mail\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'admin_mail'} eq "unconfigured") {
		print STDERR "\"admin_mail\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'retry_interval'}) {
		print STDERR "Warning: missing entry \"retry_interval\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'retry_interval'} eq "unconfigured") {
		print STDERR "\"retry_interval\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'printcap'}) {
		print STDERR "Warning: missing entry \"printcap\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'printcap'} eq "unconfigured") {
		print STDERR "\"printcap\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'verbosity'}) {
		print STDERR "Warning: missing entry \"verbosity\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'verbosity'} eq "unconfigured") {
		print STDERR "\"verbosity\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'currency_symbol'}) {
		print STDERR "Warning: missing entry \"currency_symbol\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'currency_symbol'} eq "unconfigured") {
		print STDERR "\"currency_symbol\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'bill_max_processes'}) {
		print STDERR "Warning: missing entry \"bill_max_processes\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'bill_max_processes'} eq "unconfigured") {
		print STDERR "\"bill_max_processes\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'nice'}) {
		print STDERR "Warning: missing entry \"nice\" in /etc/printbill/printbillrc\n";
		if (-x "/bin/nice") {
			$params{'nice'} = "/bin/nice";
		} elsif (-x "/usr/bin/nice") {
			$params{'nice'} = "/usr/bin/nice";
		}
	} elsif ($params{'nice'} eq "unconfigured") {
		print STDERR "\"nice\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'bill_nicethreshold'}) {
		print STDERR "Warning: missing entry \"bill_nicethreshold\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'bill_nicethreshold'} eq "unconfigured") {
		print STDERR "\"bill_nicethreshold\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'small_bill_niceness'}) {
		print STDERR "Warning: missing entry \"small_bill_niceness\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'small_bill_niceness'} eq "unconfigured") {
		print STDERR "\"small_bill_niceness\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'large_bill_niceness'}) {
		print STDERR "Warning: missing entry \"large_bill_niceness\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'large_bill_niceness'} eq "unconfigured") {
		print STDERR "\"large_bill_niceness\" not yet configured in /etc/printbill/printbillrc\n";
	}
	
	if (!defined $params{'quote_generation_niceness'}) {
		print STDERR "Warning: missing entry \"quote_generation_niceness\" in /etc/printbill/printbillrc\n";
	} elsif ($params{'quote_generation_niceness'} eq "unconfigured") {
		print STDERR "\"quote_generation_niceness\" not yet configured in /etc/printbill/printbillrc\n";
	}

	return %params;
}

# Get a list of available printers, and their configuration parameters.

sub pcfg_printers
{
	my ($config) = @_;

	my ($remote_config, %printers, $line, $subline, @strings, @pstr, $printer);
	
	$remote_config = check_remote_cfg ($config);
	
	if (defined ($remote_config)) {
		open (CONFIGFILE, $remote_config) or do {
			print STDERR "$0 (printbill_pcfg.pm): unable to open configuration file $config: $!\n";
			return ();
		};
	} elsif (-r $config) {
		open (CONFIGFILE, $config) or do {
			print STDERR "$0 (printbill_pcfg.pm): unable to open configuration file $config: $!\n";
			return ();
		}
	} else {
		return ();
	}
	
# Config file errors are ignored. The last valid entries for prices are
# accepted.
	
	while (defined ($line = <CONFIGFILE>)) {
		chomp $line;
		
		next if ($line =~ /^\#.*/ || $line eq "");

		@strings = split (/\s*:\s*/, $line);
	
		if ($#strings == 1) {
			if ($strings [0] eq 'def_colourspace' || $strings [0] eq 'def_colorspace') {
				$printers{'default'}{'colourspace'} = $strings [1];
			} elsif ($strings [0] eq 'def_price_per_percent_black') {
				$printers{'default'}{'price_per_percent_black'} = $strings [1];
			} elsif ($strings [0] eq 'def_price_per_percent_colour' || $strings [0] eq 'def_price_per_percent_color') {
				$printers{'default'}{'price_per_percent_colour'} = $strings [1];
			} elsif ($strings [0] eq 'def_price_per_page') {
				$printers{'default'}{'price_per_page'} = $strings [1];
			} elsif ($strings [0] eq 'def_estimated_total_percent_black') {
				$printers{'default'}{'estimated_total_percent_black'} = $strings [1];
			} elsif ($strings [0] eq 'def_estimated_total_percent_colour') {
				$printers{'default'}{'estimated_total_percent_colour'} = $strings [1];
			} elsif ($strings [0] eq 'defineprinter') {
				$printer = $strings [1];

				while (defined ($subline = <CONFIGFILE>) && ($subline !~ /end/i)) {
					@pstr = split (/\s*:\s*/, $subline);
					
					if ($#pstr == 1) {
						chomp $pstr [1];
						$pstr [0] =~ s/\s+//;
					
# Cater for Americans who don't speak English ;-)

						if ($pstr [0] eq 'colourspace' || $pstr [0] eq 'colorspace') {
							$printers{$printer}{'colourspace'} = $pstr [1];
						} elsif ($pstr [0] eq 'price_per_percent_black') {
							$printers{$printer}{'price_per_percent_black'} = $pstr [1];
						} elsif ($pstr [0] eq 'price_per_percent_colour' || $pstr [0] eq 'price_per_percent_color') {
							$printers{$printer}{'price_per_percent_colour'} = $pstr [1];
						} elsif ($pstr [0] eq 'price_per_page') {
							$printers{$printer}{'price_per_page'} = $pstr [1];
						} elsif ($pstr [0] eq 'estimated_total_percent_black') {
							$printers{$printer}{'estimated_total_percent_black'} = $pstr [1];
						} elsif ($pstr [0] eq 'estimated_total_percent_colour' || $pstr [0] eq 'estimated_total_percent_color') {
							$printers{$printer}{'estimated_total_percent_colour'} = $pstr [1];
						}
					}
				}
			}
		}
	}
	
# We may as well return the (apparently) valid data, even if these syscalls
# seem to have exploded.

	close CONFIGFILE or do {
		print STDERR "$0 (printbill_pcfg::pcfg_printers): unable to close config file $config: $!\n";
	};

	if (defined ($remote_config)) {
		`/bin/rm $remote_config`;
		
		if ($? >> 8) {
			print STDERR "$0 (printbill_pcfg::pcfg_printers): unable to remove remote config file $remote_config: $!\n";
		}
	}
	
	return %printers;
}

END {}

1;
